package gui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import model.Walkers;

/*
 * DrawPanel.java
 *
 * Created on 2011/07/14, 18:10:02
 * @author tadaki
 */
public class DrawPanel extends javax.swing.JPanel implements Runnable {

    private Walkers walkers;
    private volatile boolean running = false;
    private int max = 100;
    private BufferedImage image;

    public void setRunning(boolean running) {
        this.running = running;
    }

    /** Creates new form DrawPanel */
    public DrawPanel() {
        initComponents();
    }

    public void setWalkers(Walkers walkers) {
        this.walkers = walkers;
    }

    public void setMax(int max) {
        this.max = max;
    }

    private void mkImage() {
        int width = this.getPreferredSize().width;
        int height = this.getPreferredSize().height;
        image = (BufferedImage) this.createImage(width, height);
        Graphics2D g = image.createGraphics();
        g.setColor(this.getBackground());
        g.fillRect(0, 0, width, height);
        Point2D.Double f = new Point2D.Double(
                0.9 * width / max / 2., 0.8 * height / max);
        g.translate(0.5 * width, 0.9 * height);
        double w = f.x;
        Point points[] = walkers.getHistgram(max);
        g.setColor(Color.red);
        g.scale(f.x, -f.y);
        for (int i = 0; i < points.length; i++) {
            Rectangle2D.Double r =
                    new Rectangle2D.Double(points[i].x - 1, 0., 2, points[i].y);
            g.fill(r);
        }
        g = image.createGraphics();
        g.translate(0.5 * width, 0.9 * height);
        Line2D.Double lineX = new Line2D.Double(-f.x * max, 0., f.x * max, 0.);
        g.draw(lineX);
        Line2D.Double lineY = new Line2D.Double(0., 0., 0., -f.y * max);
        g.draw(lineY);
        g.setFont(this.getFont());
        g.drawString("x", (float) (f.x * max - 10.), (float) 20.);
        g.drawString("# of walkers", (float) 10., (float) (-f.y * max + 20.));
    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);
        g.drawImage(image, 0, 0, this);
    }

    @Override
    public void run() {
        while (running) {
            walkers.update();
            mkImage();
            repaint();
            try {
                Thread.sleep(100);
            } catch (InterruptedException ex) {
            }
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        setFont(new java.awt.Font("Arial", 0, 24)); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 300, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
}
