package model;

import java.awt.Point;

/**
 * 多粒子のランダムウォーク
 * @author tadaki
 */
public class Walkers {

    private int n;//粒子数
    private double p;//右への移動確率
    private Walker walkers[];//ウォーカーの配列

    /**
     * コンストラクタ
     * @param n 粒子数
     * @param p 右への移動確率
     */
    public Walkers(int n, double p) {
        this.n = n;
        this.p = p;
        walkers = new Walker[n];
        for (int i = 0; i < n; i++) {
            walkers[i] = new Walker(p);
        }
    }

    /**
     * 位置更新
     * @return 各粒子の位置 
     */
    public int[] update() {
        int x[] = new int[n];
        for (int i = 0; i < n; i++) {
            x[i] = walkers[i].update();
        }
        return x;
    }

    /**
     * 右への移動確率の変更と再初期化
     * @param p 
     */
    public void initialize(double p) {
        for (int i = 0; i < n; i++) {
            walkers[i].initialize(p, 0);
        }
    }

    /**
     * 位置に関するヒストグラムの取得
     * @param max 位置の絶対値の最大値
     * @return ヒストグラム
     */
    public Point[] getHistgram(int max) {
        int y[] = new int[2 * max + 1];
        for (int i = 0; i < n; i++){
            int x = walkers[i].getX();
            if (Math.abs(x) <= max) {
                int k = x + max;
                y[k]++;
            }
        }
        Point point[] = new Point[2 * max + 1];
        for (int i = 0; i < 2 * max + 1; i++) {
            point[i] = new Point(i - max, y[i]);
        }
        return point;
    }
}
